#include<fstream>
#include<vector>
#include<iostream>
#include<string>
#include<algorithm>
#include<stdlib.h>

#include <unistd.h>
#include "panel.hpp"
using namespace std;


static void parseArguments(int argc, char *argv[], const char *opts);
static void printUsage(char*argv[]);
static void printVersion();
static string ffilename="fi.txt"; //fi files name
static string efilename="exp_conc.txt"; //expected concentration file name
static int notes_line=6;
int main(int argc, char* argv[])
{
	
  //command-line input
  const char *opts = "hf:e:b:v";
  parseArguments(argc, argv, opts);
  
  panel p;
  
  //now we need to read the data into vectors
  vector<vector<string> > vec_fi_1;
  vector<vector<string> >vec_fi_2;
  vector<vector<string> > vec_expConc_1;
  vector<vector<string> > vec_expConc_2;

	int temp=p.read_data(ffilename, notes_line, &vec_fi_1, &vec_fi_2);
	//check for debugging
	
		if(temp!=0)
		{
			exit(-1);
		}

	cout<<"==================="<<endl;
	cout<<"Summary for data reading:"<<endl;
	cout<<"\tone file read in, \""<<ffilename<<"\""<<endl;
		cout<<"\t"<< vec_fi_1.size()+vec_fi_2.size()<<" lines read"<<endl;
		//cout<<"content is "<<vec_fi_1[0][vec_fi_1[0].size()-1]<<endl;
		//cout<<"vec2 size is "<<vec_fi_2.size()<<endl;
		cout<<"===================\n"<<endl;		
	
	temp=p.read_data(efilename, notes_line, &vec_expConc_1, &vec_expConc_2);
	if(temp!=0)
		{
			exit(-1);
		}
	
	cout<<"==================="<<endl;
	cout<<"Summary for data reading:"<<endl;
	cout<<"\tone file read in, \""<<ffilename<<"\""<<endl;
	cout<<"\t"<< vec_expConc_1.size()+vec_expConc_2.size()<<" lines read"<<endl;
		//cout<<"content is "<<vec_fi_1[0][vec_fi_1[0].size()-1]<<endl;
	cout<<"===================\n"<<endl;
	
		//now we are ready to parse the information
		cout<<"Starting parsing the data..........."<<endl;
		if(!p.check_analyte_name(vec_fi_1, vec_fi_2, vec_expConc_1, vec_expConc_2))
			{
				cout<<"******ERROR:the analyte names are not consistent, please check the file ......"<<endl;
				exit(-1);
			}
		vector<string> vec_name;
		vec_name=p.get_analyte_name(vec_fi_1);
		
		//cout<<"name vec size is "<< vec_name.size()<<endl;	
			
		//now we test reading of calibration data
		vector<vector<string> >out_cal_conc;
		vector<vector<string> >out_cal_fi;
		
		p.read_cal_data(vec_fi_2, vec_expConc_1, &out_cal_conc, &out_cal_fi); 
		/*cout<<"out cal conc vec size is "<< out_cal_conc.size()<<endl;
		cout<<"out cal fi vec size is "<< out_cal_fi.size()<<endl;
		
		cout<<"***************"<<endl;
		for(unsigned int i=0;i<out_cal_conc.size();i++)
		{
			for(unsigned int j=0;j<out_cal_conc[i].size();j++)
			{
				cout<<out_cal_conc[i][j]<<";";
			}
			cout<<endl;	
		}
		
		cout<<"***************"<<endl;
		for(unsigned int i=0;i<out_cal_fi.size();i++)
		{
			for(unsigned int j=0;j<out_cal_fi[i].size();j++)
			{
				cout<<out_cal_fi[i][j]<<";";
			}
			cout<<endl;	
		}
		cout<<endl;
		*/
		//now testing the read target
		vector<vector<string> > out_target;
		p.read_target_data(vec_fi_1,vec_fi_2, &out_target);
		/*cout<<"out target size is "<< out_target.size()<<endl;
		
		cout<<"%%%%%%%%%%%%%%%%%%%"<<endl;
		for(unsigned int i=0;i<out_target.size();i++)
		{
			for(unsigned int j=0;j<out_target[i].size();j++)
			{
				cout<<out_target[i][j]<<";";
			}
			cout<<endl;	
		}
		cout<<endl;
		*/
		cout<<"Done.........."<<endl;
		
	//write the output
	cout<<"Writing the output files to the disk.............."<<endl;
	p.write_cal_data(vec_name, out_cal_conc, out_cal_fi);	
	
	p.write_target_data(vec_name, out_target);
	//clean up
		
		
	cout<<"Done........"<<endl;
	return 0;

}




static void parseArguments(int argc, char * argv[], const char *opts)
{
  int optchar;

  while ((optchar = getopt(argc, argv, opts)) != -1)
    {
      switch(char(optchar))
			{
			case 'f':
			  //cout<<"optarg "<<optarg<<endl;
			  ffilename = optarg;
			  //targetMode=1;
			  break;
			  
			case 'e':
			  efilename = optarg;
			  break;
			case 'b':
			  notes_line=atoi(optarg);
			  break;
			 
			case 'v':
			  printVersion();
			  break;
			
		  case '?':
			  
			    if(isprint(optopt))
			    {
			      cout<<"Unknown option "<<optopt<<endl;
			    }
			  else
			    cout<<"Unknown option character"<<endl;
			  
			case 'h': // usage or unknown
			  
			default:
			  printUsage(argv);
			  exit(1);
			}
    }
    
}

static void printUsage(char* argv[])
{
  cout << "Usage: "<<argv[0]<<" [-f <fi file>] \n"
       << "             [-e <expected concentration file>] [-b <number of lines before the header of the tables>] \n"
       //<< "           [-m #][-b #]\n"
       << "             [-h][-v]\n"
       << "                \n";

  cout << "   -f <fi file>  -- the name of the file containing the FI value, \"fi.txt\" by default."
       << "\n";
  cout << "   -e <expected concentration file>  -- the expected concentration input file, \"exp_conc.txt\" by default"
       << "\n";
  //cout << "   -b #  -- the number of lines for metadata or notations before the table contains"
  //		 <<" the input values, 5 by default"
  //     << "\n";
  
  cout << "   -v   -- print the Version information"
       << "\n";
  cout << "   -h  -- print the help information"
       << "\n";
  cout << "\n";
}

static void printVersion()
{
  cout<<"Version Information:\n"
      <<"\tgenerate the input files from luminex data files\n"
      <<"\t Ver: 1.0\n"
      <<"\t By Feng Feng and Thomas Kepler @ Center for Computational Immunology, Duke University\n"
      <<"\t Dec, 2010\n"
      <<"\t Bug report to Feng Feng (ff11@duke.edu)\n";
  exit(0);
}

